@@ -890,7 +890,7 @@ static int parse(pTHX_ OP **op, struct Registration *reg)
     ret = (*hooks->build1)(aTHX_ op, args + 0, reg->hookdata);
 
   if(is_blockscope)
-    *op = block_end(floor, *op);
+    *op = op_scope(block_end(floor, *op));
 
   switch(hooks->flags & (XPK_FLAG_EXPR|XPK_FLAG_STMT)) {
     case XPK_FLAG_EXPR:
